@@ -1,6 +1,6 @@
 /*
  * Carla common defines
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2021 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,12 +32,12 @@
 #endif
 
 /* Set Version */
-#define CARLA_VERSION_HEX    0x020300
-#define CARLA_VERSION_STRING "2.3.0"
-#define CARLA_VERSION_STRMIN "2.3"
+#define CARLA_VERSION_HEX    0x020401
+#define CARLA_VERSION_STRING "2.4.1"
+#define CARLA_VERSION_STRMIN "2.4"
 
 /* Check OS */
-#if defined(WIN64) || defined(_WIN64) || defined(__WIN64__)
+#if defined(WIN64) || defined(_WIN64) || defined(__WIN64__) || defined(_M_ARM64)
 # define CARLA_OS_WIN64
 #elif defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
 # define CARLA_OS_WIN32
@@ -61,7 +61,7 @@
 # define CARLA_OS_UNIX
 #endif
 
-#if defined (__LP64__) || defined (_LP64) || defined (__arm64__) || defined (__aarch64__) || defined(CARLA_OS_WIN64)
+#if defined(__LP64__) || defined(_LP64) || defined(__arm64__) || defined(__aarch64__) || defined(CARLA_OS_WIN64)
 # define CARLA_OS_64BIT
 #endif
 
@@ -270,7 +270,7 @@ private:                                        \
 #endif
 
 /* Define CARLA_API */
-#ifdef BUILD_BRIDGE
+#if defined(BUILD_BRIDGE) || defined(STATIC_PLUGIN_TARGET)
 # define CARLA_API
 #else
 # if defined(CARLA_OS_WIN) && ! defined(__WINE__)
