@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Common Carla code
-# Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+# Copyright (C) 2011-2021 Filipe Coelho <falktx@falktx.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -16,13 +16,6 @@
 #
 # For a full copy of the GNU General Public License see the doc/GPL.txt file.
 
-# ------------------------------------------------------------------------------------------------------------
-# Config
-
-# These will be modified during install
-X_LIBDIR_X = "/usr/lib"
-X_DATADIR_X = "/usr/share"
-
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Global)
 
@@ -31,13 +24,8 @@ import sys
 
 from math import fmod
 
-from PyQt5.Qt import PYQT_VERSION_STR
-from PyQt5.QtCore import qFatal, QT_VERSION, QT_VERSION_STR, qWarning, QDir, QSettings
-from PyQt5.QtGui import QIcon
-from PyQt5.QtWidgets import QFileDialog, QMessageBox
-
 # ------------------------------------------------------------------------------------------------------------
-# Import Signal
+# Imports (Signal)
 
 from signal import signal, SIGINT, SIGTERM
 
@@ -47,10 +35,32 @@ try:
 except:
     haveSIGUSR1 = False
 
+# ------------------------------------------------------------------------------------------------------------
+# Imports (PyQt5)
+
+from PyQt5.Qt import PYQT_VERSION_STR
+from PyQt5.QtCore import qFatal, QT_VERSION, QT_VERSION_STR, qWarning, QDir, QSettings
+from PyQt5.QtGui import QIcon
+from PyQt5.QtWidgets import QFileDialog, QMessageBox
+
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Custom)
 
-from carla_backend_qt import *
+from carla_backend import (
+    kIs64bit, HAIKU, LINUX, MACOS, WINDOWS,
+    MAX_DEFAULT_PARAMETERS,
+    ENGINE_PROCESS_MODE_MULTIPLE_CLIENTS,
+    ENGINE_PROCESS_MODE_PATCHBAY,
+    ENGINE_TRANSPORT_MODE_INTERNAL,
+    ENGINE_TRANSPORT_MODE_JACK
+)
+
+# ------------------------------------------------------------------------------------------------------------
+# Config
+
+# These will be modified during install
+X_LIBDIR_X = "/usr/lib"
+X_DATADIR_X = "/usr/share"
 
 # ------------------------------------------------------------------------------------------------------------
 # Platform specific stuff
@@ -61,7 +71,7 @@ if WINDOWS:
 # ------------------------------------------------------------------------------------------------------------
 # Set Version
 
-VERSION = "2.1.1"
+VERSION = "2.4.1"
 
 # ------------------------------------------------------------------------------------------------------------
 # Set TMP
@@ -126,7 +136,7 @@ MIDI_CC_LIST = (
     "04 [0x04] Foot",
     "05 [0x05] Portamento",
     "07 [0x07] Volume",
-    "02 [0x08] Balance",
+    "08 [0x08] Balance",
     "10 [0x0A] Pan",
     "11 [0x0B] Expression",
     "12 [0x0C] FX Control 1",
@@ -173,7 +183,9 @@ CARLA_KEY_MAIN_USE_PRO_THEME    = "Main/UseProTheme"     # bool
 CARLA_KEY_MAIN_PRO_THEME_COLOR  = "Main/ProThemeColor"   # str
 CARLA_KEY_MAIN_REFRESH_INTERVAL = "Main/RefreshInterval" # int
 CARLA_KEY_MAIN_CONFIRM_EXIT     = "Main/ConfirmExit"     # bool
+CARLA_KEY_MAIN_CLASSIC_SKIN     = "Main/ClassicSkin"     # bool
 CARLA_KEY_MAIN_SHOW_LOGS        = "Main/ShowLogs"        # bool
+CARLA_KEY_MAIN_SYSTEM_ICONS     = "Main/SystemIcons"     # bool
 CARLA_KEY_MAIN_EXPERIMENTAL     = "Main/Experimental"    # bool
 
 CARLA_KEY_CANVAS_THEME             = "Canvas/Theme"           # str
@@ -248,7 +260,9 @@ CARLA_DEFAULT_MAIN_USE_PRO_THEME    = True
 CARLA_DEFAULT_MAIN_PRO_THEME_COLOR  = "Black"
 CARLA_DEFAULT_MAIN_REFRESH_INTERVAL = 20
 CARLA_DEFAULT_MAIN_CONFIRM_EXIT     = True
+CARLA_DEFAULT_MAIN_CLASSIC_SKIN     = False
 CARLA_DEFAULT_MAIN_SHOW_LOGS        = bool(not WINDOWS)
+CARLA_DEFAULT_MAIN_SYSTEM_ICONS     = False
 CARLA_DEFAULT_MAIN_EXPERIMENTAL     = False
 
 # Canvas
@@ -298,7 +312,7 @@ else:
     CARLA_DEFAULT_TRANSPORT_MODE = ENGINE_TRANSPORT_MODE_INTERNAL
 
 # OSC
-CARLA_DEFAULT_OSC_ENABLED = not WINDOWS
+CARLA_DEFAULT_OSC_ENABLED = not (MACOS or WINDOWS)
 CARLA_DEFAULT_OSC_TCP_PORT_ENABLED = True
 CARLA_DEFAULT_OSC_TCP_PORT_NUMBER  = 22752
 CARLA_DEFAULT_OSC_TCP_PORT_RANDOM  = False
@@ -456,7 +470,7 @@ else:
     DEFAULT_VST3_PATH   += ":/usr/local/lib/vst3"
 
     DEFAULT_SF2_PATH     = HOME + "/.sounds/sf2"
-    DEFAULT_SF2_PATH    += HOME + "/.sounds/sf3"
+    DEFAULT_SF2_PATH    += HOME + ":/.sounds/sf3"
     DEFAULT_SF2_PATH    += ":/usr/share/sounds/sf2"
     DEFAULT_SF2_PATH    += ":/usr/share/sounds/sf3"
     DEFAULT_SF2_PATH    += ":/usr/share/soundfonts"
@@ -487,19 +501,21 @@ readEnvVars = True
 
 if WINDOWS:
     # Check if running Wine. If yes, ignore env vars
+    # pylint: disable=import-error
     from winreg import ConnectRegistry, OpenKey, CloseKey, HKEY_CURRENT_USER
-    reg = ConnectRegistry(None, HKEY_CURRENT_USER)
+    # pylint: enable=import-error
+    _reg = ConnectRegistry(None, HKEY_CURRENT_USER)
 
     try:
-        key = OpenKey(reg, r"SOFTWARE\Wine")
-        CloseKey(key)
-        del key
+        _key = OpenKey(_reg, r"SOFTWARE\Wine")
+        CloseKey(_key)
+        del _key
         readEnvVars = False
     except:
         pass
 
-    CloseKey(reg)
-    del reg
+    CloseKey(_reg)
+    del _reg
 
 if readEnvVars:
     CARLA_DEFAULT_LADSPA_PATH = os.getenv("LADSPA_PATH", DEFAULT_LADSPA_PATH).split(splitter)
@@ -533,19 +549,15 @@ del DEFAULT_SFZ_PATH
 # ------------------------------------------------------------------------------------------------------------
 # Global Carla object
 
-class CarlaObject(object):
-    __slots__ = [
-        'gui',   # Host Window
-        'nogui', # Skip UI
-        'term',  # Terminated by OS signal
-        'utils'  # Utils object
-    ]
+class CarlaObject():
+    def __init__(self):
+        self.cnprefix = ""    # Client name prefix
+        self.gui      = None  # Host Window
+        self.nogui    = False # Skip UI
+        self.term     = False # Terminated by OS signal
+        self.utils    = None  # Utils object
 
 gCarla = CarlaObject()
-gCarla.gui   = None
-gCarla.nogui = False
-gCarla.term  = False
-gCarla.utils = None
 
 # ------------------------------------------------------------------------------------------------------------
 # Set CWD
@@ -558,7 +570,11 @@ if not CWD:
 # make it work with cxfreeze
 if os.path.isfile(CWD):
     CWD = os.path.dirname(CWD)
+    if CWD.endswith("/lib"):
+        CWD = CWD.rsplit("/lib",1)[0]
     CXFREEZE = True
+    if not WINDOWS:
+        os.environ['CARLA_MAGIC_FILE'] = os.path.join(CWD, "magic.mgc")
 else:
     CXFREEZE = False
 
@@ -582,8 +598,8 @@ def countDecimalPoints(step, stepSmall):
         return 2
 
     count = 0
-    value = fmod(abs(stepSmall), 1)
-    while value > 0.001 and value < 0.99 and count < 6:
+    value = fmod(abs(step), 1)
+    while 0.0001 < value < 0.999 and count < 6:
         value = fmod(value*10, 1)
         count += 1
 
@@ -605,16 +621,15 @@ def isNumber(value):
 def toList(value):
     if value is None:
         return []
-    elif not isinstance(value, list):
+    if not isinstance(value, list):
         return [value]
-    else:
-        return value
+    return value
 
 # ------------------------------------------------------------------------------------------------------------
 # Get Icon from user theme, using our own as backup (Oxygen)
 
-def getIcon(icon, size = 16):
-    return QIcon.fromTheme(icon, QIcon(":/%ix%i/%s.png" % (size, size, icon)))
+def getIcon(icon, size, qrcformat):
+    return QIcon.fromTheme(icon, QIcon(":/%ix%i/%s.%s" % (size, size, icon, qrcformat)))
 
 # ------------------------------------------------------------------------------------------------------------
 # Handle some basic command-line arguments shared between all carla variants
@@ -622,6 +637,7 @@ def getIcon(icon, size = 16):
 def handleInitialCommandLineArguments(file):
     initName  = os.path.basename(file) if (file is not None and os.path.dirname(file) in PATH) else sys.argv[0]
     libPrefix = None
+    readPrefixNext = False
 
     for arg in sys.argv[1:]:
         if arg.startswith("--with-appname="):
@@ -630,12 +646,24 @@ def handleInitialCommandLineArguments(file):
         elif arg.startswith("--with-libprefix="):
             libPrefix = arg.replace("--with-libprefix=", "")
 
+        elif arg.startswith("--osc-gui="):
+            gCarla.nogui = int(arg.replace("--osc-gui=", ""))
+
+        elif arg.startswith("--cnprefix="):
+            gCarla.cnprefix = arg.replace("--cnprefix=", "")
+
+        elif arg == "--cnprefix":
+            readPrefixNext = True
+
         elif arg == "--gdb":
             pass
 
         elif arg in ("-n", "--n", "-no-gui", "--no-gui", "-nogui", "--nogui"):
             gCarla.nogui = True
 
+        elif MACOS and arg.startswith("-psn_"):
+            pass
+
         elif arg in ("-h", "--h", "-help", "--help"):
             print("Usage: %s [OPTION]... [FILE|URL]" % initName)
             print("")
@@ -643,13 +671,20 @@ def handleInitialCommandLineArguments(file):
             print("")
             print(" and OPTION can be one or more of the following:")
             print("")
-            print("    --gdb    \t Run Carla inside gdb.")
-            print(" -n,--no-gui \t Run Carla headless, don't show UI.")
-            print("")
-            print(" -h,--help   \t Print this help text and exit.")
-            print(" -v,--version\t Print version information and exit.")
+            print("    --cnprefix\t Set a prefix for client names in multi-client mode.")
+            if isinstance(gCarla.nogui, bool):
+                if X_LIBDIR_X is not None:
+                    print("    --gdb     \t Run Carla inside gdb.")
+                print(" -n,--no-gui  \t Run Carla headless, don't show UI.")
+                print("")
+            print(" -h,--help    \t Print this help text and exit.")
+            print(" -v,--version \t Print version information and exit.")
             print("")
 
+            if not isinstance(gCarla.nogui, bool):
+                print("NOTE: when using %s the FILE is only valid the first time the backend is started" % initName)
+                sys.exit(1)
+
             sys.exit(0)
 
         elif arg in ("-v", "--v", "-version", "--version"):
@@ -662,22 +697,48 @@ def handleInitialCommandLineArguments(file):
             print("  Binary dir:     %s" % pathBinaries)
             print("  Resources dir:  %s" % pathResources)
 
-            sys.exit(0)
+            sys.exit(1 if gCarla.nogui else 0)
+
+        elif readPrefixNext:
+            readPrefixNext = False
+            gCarla.cnprefix = arg
+
+    if gCarla.nogui and not isinstance(gCarla.nogui, bool):
+        if os.fork():
+            # pylint: disable=protected-access
+            os._exit(0)
+            # pylint: enable=protected-access
+        else:
+            os.setsid()
 
     return (initName, libPrefix)
 
 # ------------------------------------------------------------------------------------------------------------
 # Get initial project file (as passed in the command-line parameters)
 
-def getInitialProjectFile(app, skipExistCheck = False):
-    # FIXME - PyQt mishandles unicode characters, we'll use direct sys.argv for now
+def getInitialProjectFile(skipExistCheck = False):
+    # NOTE: PyQt mishandles unicode characters, we directly use sys.argv instead of qApp->arguments()
     # see https://riverbankcomputing.com/pipermail/pyqt/2015-January/035395.html
-    #args = app.arguments()[1:]
     args = sys.argv[1:]
+    readPrefixNext = False
     for arg in args:
-        if arg.startswith("--with-appname=") or arg.startswith("--with-libprefix=") or arg == "--gdb":
+        if readPrefixNext:
+            readPrefixNext = False
             continue
-        if arg in ("-n", "--n", "-no-gui", "--no-gui", "-nogui", "--nogui"):
+        if arg.startswith("--cnprefix="):
+            continue
+        if arg.startswith("--osc-gui="):
+            continue
+        if arg.startswith("--with-appname="):
+            continue
+        if arg.startswith("--with-libprefix="):
+            continue
+        if arg == "--cnprefix":
+            readPrefixNext = True
+            continue
+        if arg in ("-n", "--n", "-no-gui", "--no-gui", "-nogui", "--nogui", "--gdb"):
+            continue
+        if MACOS and arg.startswith("-psn_"):
             continue
         arg = os.path.expanduser(arg)
         if skipExistCheck or os.path.exists(arg):
@@ -765,7 +826,7 @@ def getAndSetPath(parent, lineEdit):
 # Backwards-compatible horizontalAdvance/width call, depending on Qt version
 
 def fontMetricsHorizontalAdvance(fontMetrics, string):
-    if QT_VERSION >= 0x51100:
+    if QT_VERSION >= 0x50b00:
         return fontMetrics.horizontalAdvance(string)
     return fontMetrics.width(string)
 
@@ -781,7 +842,7 @@ class QMessageBoxWithBetterWidth(QMessageBox):
 
         lines = self.text().strip().split("\n") + self.informativeText().strip().split("\n")
 
-        if len(lines) > 0:
+        if lines:
             width = 0
 
             for line in lines:
@@ -795,18 +856,19 @@ class QMessageBoxWithBetterWidth(QMessageBox):
 # Safer QSettings class, which does not throw if type mismatches
 
 class QSafeSettings(QSettings):
-    def value(self, key, defaultValue, type):
-        if not isinstance(defaultValue, type):
+    def value(self, key, defaultValue, valueType):
+        if not isinstance(defaultValue, valueType):
             print("QSafeSettings.value() - defaultValue type mismatch for key", key)
 
         try:
-            return QSettings.value(self, key, defaultValue, type)
+            return QSettings.value(self, key, defaultValue, valueType)
         except:
             return defaultValue
 
 # ------------------------------------------------------------------------------------------------------------
 # Custom MessageBox
 
+# pylint: disable=too-many-arguments
 def CustomMessageBox(parent, icon, title, text,
                      extraText="",
                      buttons=QMessageBox.Yes|QMessageBox.No,
@@ -819,5 +881,6 @@ def CustomMessageBox(parent, icon, title, text,
     msgBox.setStandardButtons(buttons)
     msgBox.setDefaultButton(defButton)
     return msgBox.exec_()
+# pylint: enable=too-many-arguments
 
 # ------------------------------------------------------------------------------------------------------------
